"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _base = require("./base");

var _json = require("./json");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
class HtmlReporter extends _base.BaseReporter {
  async onEnd() {
    const targetFolder = process.env[`PLAYWRIGHT_HTML_REPORT`] || 'playwright-report';

    _fs.default.mkdirSync(targetFolder, {
      recursive: true
    });

    const appFolder = _path.default.join(__dirname, '..', '..', 'web', 'htmlReport');

    for (const file of _fs.default.readdirSync(appFolder)) _fs.default.copyFileSync(_path.default.join(appFolder, file), _path.default.join(targetFolder, file));

    const stats = {
      expected: 0,
      unexpected: 0,
      skipped: 0,
      flaky: 0
    };

    const reportFile = _path.default.join(targetFolder, 'report.json');

    const output = {
      config: { ...this.config,
        rootDir: (0, _json.toPosixPath)(this.config.rootDir),
        projects: this.config.projects.map(project => {
          return {
            outputDir: (0, _json.toPosixPath)(project.outputDir),
            repeatEach: project.repeatEach,
            retries: project.retries,
            metadata: project.metadata,
            name: project.name,
            testDir: (0, _json.toPosixPath)(project.testDir),
            testIgnore: (0, _json.serializePatterns)(project.testIgnore),
            testMatch: (0, _json.serializePatterns)(project.testMatch),
            timeout: project.timeout
          };
        })
      },
      stats,
      suites: this.suite.suites.map(s => this._serializeSuite(s))
    };

    _fs.default.writeFileSync(reportFile, JSON.stringify(output));
  }

  _relativeLocation(location) {
    if (!location) return {
      file: '',
      line: 0,
      column: 0
    };
    return {
      file: (0, _json.toPosixPath)(_path.default.relative(this.config.rootDir, location.file)),
      line: location.line,
      column: location.column
    };
  }

  _serializeSuite(suite) {
    return {
      title: suite.title,
      location: this._relativeLocation(suite.location),
      suites: suite.suites.map(s => this._serializeSuite(s)),
      tests: suite.tests.map(t => this._serializeTest(t))
    };
  }

  _serializeTest(test) {
    return {
      title: test.title,
      location: this._relativeLocation(test.location),
      expectedStatus: test.expectedStatus,
      timeout: test.timeout,
      annotations: test.annotations,
      retries: test.retries,
      ok: test.ok(),
      outcome: test.outcome(),
      results: test.results.map(r => this._serializeResult(test, r))
    };
  }

  _serializeResult(test, result) {
    return {
      retry: result.retry,
      workerIndex: result.workerIndex,
      startTime: result.startTime.toISOString(),
      duration: result.duration,
      status: result.status,
      error: result.error,
      failureSnippet: (0, _base.formatResultFailure)(test, result, '').join('') || undefined,
      attachments: result.attachments,
      stdout: result.stdout,
      stderr: result.stderr,
      steps: this._serializeSteps(result.steps)
    };
  }

  _serializeSteps(steps) {
    const stepStack = [];
    const result = [];
    const stepMap = new Map();

    for (const step of steps) {
      var _stepMap$get;

      let lastStep = stepStack[stepStack.length - 1];

      while (lastStep && !containsStep(lastStep, step)) {
        stepStack.pop();
        lastStep = stepStack[stepStack.length - 1];
      }

      const collection = ((_stepMap$get = stepMap.get(lastStep)) === null || _stepMap$get === void 0 ? void 0 : _stepMap$get.steps) || result;
      const jsonStep = {
        title: step.title,
        category: step.category,
        startTime: step.startTime.toISOString(),
        duration: step.duration,
        error: step.error,
        steps: []
      };
      collection.push(jsonStep);
      stepMap.set(step, jsonStep);
      stepStack.push(step);
    }

    return result;
  }

}

function containsStep(outer, inner) {
  if (outer.startTime.getTime() > inner.startTime.getTime()) return false;
  if (outer.startTime.getTime() + outer.duration < inner.startTime.getTime() + inner.duration) return false;
  if (outer.startTime.getTime() + outer.duration <= inner.startTime.getTime()) return false;
  return true;
}

var _default = HtmlReporter;
exports.default = _default;